(function($) {

  /**
   * Font loader that checks for font existence.
   *
   * For the font detection method see:
   * https://samclarke.com/2013/06/javascript-is-font-available/
   *
   * @class
   */
  Contextly.FontManager = Contextly.createClass({

    extend: [Contextly.ProxyEach.prototype, Contextly.Transmitter.prototype],

    statics: /** @lends Contextly.FontManager */ {

      construct: function() {
        this.left = 0;
        this.fonts = {};
        this.callbacks = [];
        this.fallbackWidths = {};
        this.scheduled = [];
      },

      getContainerStyle: function(fonts) {
        var style = 'position: absolute !important; white-space: nowrap !important; font-size: 128px !important; width: auto !important; top: -9999px !important; left: 0 !important;';

        if (fonts && fonts.length) {
          // Quote and cleanup font names. We don't follow any escaping specs
          // here, just drop everything except for alphanumeric, space, - and _.
          var cssFonts = [];
          for (var i = 0; i < fonts.length; i++) {
            cssFonts.push('"' + fonts[i].replace(/[^a-z0-9\-\_\ ]/ig, '') + '"');
          }

          style += ' font-family: ' + cssFonts.join(',') + ' !important';
        }
        else {
          style += ' display: none !important';
        }

        return style;
      },

      getContainer: function() {
        if (typeof this.container === 'undefined') {
          this.container = $('<div/>')
            .attr('style', this.getContainerStyle())
            .text((new Array(40)).join('wmkgil'))
            .appendTo('body');
        }

        return this.container;
      },

      getFontWidth: function(font, fallback) {
        var fonts = [font];
        if (fallback) {
          fonts.push(fallback);
        }

        var container = this.getContainer()
          .attr('style', this.getContainerStyle(fonts));
        var width = container.width();
        container.attr('style', this.getContainerStyle());

        return width;
      },

      getFallbackFonts: function() {
        return ['serif', 'sans-serif', 'monospace'];
      },

      fontExists: function(font) {
        var fallbacks = this.getFallbackFonts();
        for (var i = 0; i < fallbacks.length; i++) {
          var fallback = fallbacks[i];

          if (typeof this.fallbackWidths[fallback] === 'undefined') {
            this.fallbackWidths[fallback] = this.getFontWidth(fallback);
          }

          if (this.getFontWidth(font, fallback) !== this.fallbackWidths[fallback]) {
            return true;
          }
        }

        return false;
      },

      onFontFailed: function(font) {
        this.left--;
        this.fonts[font] = this.fontStates.FAILED;
        this.broadcast(this.broadcastTypes.FONT_FAILED, font);
        this.onLoadingComplete();
      },

      onFontLoaded: function(font) {
        this.left--;
        this.fonts[font] = this.fontStates.LOADED;
        this.broadcast(this.broadcastTypes.FONT_LOADED, font);
        this.onLoadingComplete();
      },

      onLoadingComplete: function() {
        if (this.left > 0) {
          return;
        }

        this.each(this.callbacks, function(item) {
          var fonts = item.fonts;
          var callback = item.callback;
          var states = {};
          this.each(fonts, function(font) {
            states[font] = this.fonts[font];
          });

          try {
            callback(states);
          }
          catch (e) {
            Contextly.Utils.logError('FontLoader callback failed.', item, e);
          }
        });

        this.broadcast(this.broadcastTypes.LOADED);
      },

      scheduleLoading: function(s) {
        // As soon as the loading has been scheduled with any of methods, we
        // don't care
        var isScheduled = !!this.scheduled.length;
        this.scheduled.push(s);
        if (isScheduled) {
          return;
        }

        if (window.WebFont) {
          // WebFontLoader is here, just postpone a bit to process all at once.
          setTimeout(this.proxy(this.loadScheduled), 0);
        }
        else if (window.WebFontConfig) {
          // There is a WebFontLoader config, so the library must be on its way,
          // as it could only happen in case client forced our library manager
          // to not load this lib due to async loading for their own purposes.

          // First make sure events haven't been disabled.
          if (window.WebFontConfig.events === false) {
            Contexyly.Utils.logError('Unable to load fonts required for the Contextly module because events are disabled on the WebFontConfig.');
            return;
          }

          // Inject our handler into the "loading" event to load scheduled
          // configs when WebFontLoader arrives. It seems to trigger this event
          // even on no fonts provided in config.
          var originalCallback = window.WebFontConfig.loading;
          var ourCallback = this.proxy(this.loadScheduled);
          window.WebFontConfig.loading = function() {
            // Postpone a little bit more, just to make sure we don't do weird
            // things in WebFontLoader library.
            setTimeout(ourCallback, 0);

            if (originalCallback) {
              return originalCallback.apply(this, arguments);
            }
          };
        }
        else {
          Contexyly.Utils.logError('Unable to load fonts required for the Contextly module because of missing WebFontLoader.');
        }
      },

      loadScheduled: function() {
        while (this.scheduled.length) {
          var s = this.scheduled.shift();
          this.startLoading(s);
        }
      },

      startLoading: function(s) {
        if (this.left == 0) {
          this.broadcast(this.broadcastTypes.LOADING);
        }

        this.left += s.fonts.length;
        for (var i = 0; i < s.fonts.length; i++) {
          var font = s.fonts[i];
          this.fonts[font] = this.fontStates.LOADING;
          this.broadcast(this.broadcastTypes.FONT_LOADING, font);
        }

        var config = {
          classes: false,
          fontactive: this.proxy(this.onFontLoaded, false, true),
          fontinactive: this.proxy(this.onFontFailed, false, true)
        };
        config[s.provider] = $.extend({}, s.options, {
          families: s.fonts
        });
        WebFont.load(config);
      },

      load: function(specs) {
        var s = $.extend({
          provider: 'google',
          fonts: [],
          callback: null,
          options: {}
        }, specs);

        if (typeof s.callback === 'function') {
          this.callbacks.push({
            fonts: s.fonts,
            callback: s.callback
          });
        }

        var load = [];
        for (var i = 0; i < s.fonts.length; i++) {
          var font = s.fonts[i];

          if (typeof this.fonts[font] !== 'undefined') {
            continue;
          }

          if (this.fontExists(font)) {
            this.fonts[font] = this.fontStates.EXISTS;
            continue;
          }

          load.push(font);
        }

        if (load.length) {
          s.fonts = load;
          this.scheduleLoading(s);
        }
        else {
          this.onLoadingComplete();
        }
      }

    }

  });

  Contextly.FontManager.fontStates = {
    LOADING: 'loading',
    LOADED: 'loaded',
    EXISTS: 'exists',
    FAILED: 'failed'
  };

  Contextly.FontManager.broadcastTypes = {
    LOADING: 'contextlyFontsLoading',
    LOADED: 'contextlyFontsLoaded',
    FONT_LOADING: 'contextlyFontLoading',
    FONT_LOADED: 'contextlyFontLoaded',
    FONT_FAILED: 'contextlyFontFailed'
  };

})(jQuery);

