(function () {

  /**
   * @class
   */
  Contextly.Utils = Contextly.createClass({

    statics: /** @lends Contextly.Utils */ {

      escape: function(text) {
        return (text + '')
          .replace(/&/g, "&amp;")
          .replace(/</g, "&lt;")
          .replace(/>/g, "&gt;")
          .replace(/"/g, "&quot;")
          .replace(/'/g, "&#039;");
      },

      /**
       * Removes some tags and decodes some HTML entities.
       *
       * Should *not* be used for security purposes, as it uses not really safe
       * regular expression to do its job!
       */
      htmlToText: function(html) {
        var map = {
          "amp": "&",
          "lt": "<",
          "gt": ">",
          "quot": '"',
          "#039": "'",
          "apos": "'",
          "nbsp": " "
        };

        return (html + '')
          .replace(/<[a-z0-9\/][^>]*>/ig, '')
          .replace(/&(#[0-9]+|[a-z]+);/ig, function(match, p1) {
            if (map[p1] != null) {
              return map[p1];
            }
            else {
              return match;
            }
          });
      },

      escapeSizzleAttrValue: function(value) {
        if (!value) {
          return '';
        }

        // We should probably only follow string escaping from the specs, but
        // jQuery doesn't really understand the specs, so we combine string
        // escaping with identifier escaping.
        // See https://drafts.csswg.org/cssom/#common-serializing-idioms
        // So, we escape everything except for: U+0080, "-" (U+002D), "_"
        // (U+005F), [0-9] (U+0030 to U+0039), [A-Z] (U+0041 to U+005A),
        // [a-z] (U+0061 to U+007A).
        return value.replace(/[^\u0080\u002D\u005F\u0030-\u0039\u0041-\u005A\u0061-\u007A]/g, function(found) {
          var code = found.charCodeAt(0);
          if (code === 0x0000) {
            return '\\' + '00FFFD';
          }
          else if (code >= 0x0001 && code <= 0x001F) {
            return '\\' + code.toString(16) + ' ';
          }
          else {
            return '\\' + found;
          }
        });
      },

      /**
       * Writes an error to the browser console, if any.
       */
      logError: function() {
        if (window.console && typeof console.error === 'function') {
          console.error.apply(console, arguments);
        }
      },

      isEmailValid: function (email) {
        var re = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
        return re.test(email);
      },

      /**
       * Returns true if passed object is an array.
       *
       * @see http://perfectionkills.com/instanceof-considered-harmful-or-how-to-write-a-robust-isarray/
       */
      isArray: function(o) {
        return Object.prototype.toString.call(o) === '[object Array]';
      },

      /**
       * Returns true if passed variable is string.
       *
       * @see http://stackoverflow.com/a/9436948/404521
       */
      isString: function(o) {
        return (typeof o === 'string' || o instanceof String);
      },

      isFunction: function(o) {
        return (typeof o === 'function');
      },

      isObject: function(o) {
        return (typeof o === 'object');
      },

      /**
       * Merge all properties from passed objects into a new one.
       */
      merge: function() {
        var result = {};
        for (var i = 0; i < arguments.length; i++) {
          var obj = arguments[i];
          for (var k in obj) {
            result[k] = obj[k];
          }
        }
        return result;
      },

      /**
       * Returns true if the passed variable is an empty object.
       */
      isEmptyObject: function(o) {
        for (var key in o) {
          return false;
        }
        return true;
      },

      error: function(msg) {
        throw msg;
      },

      escapeRegexp: function(str) {
        if (!str) {
          return '';
        }

        // http://stackoverflow.com/a/6969486/404521
        return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
      },

      parseUrlQueryParameter: function(key, url) {
        var expression = "[\\?&]" + Contextly.Utils.escapeRegexp(key) + "\=([^&#]*)";
        var regexp = new RegExp(expression);
        var results = regexp.exec(url);
        if (results == null) {
          return '';
        }
        else {
          return results[1];
        }
      },

      addHeadElement: function(element) {
        // We can't rely on head existence here,
        // see http://www.paulirish.com/2011/surefire-dom-element-insertion/
        var ref = document.getElementsByTagName('script')[0];
        ref.parentNode.insertBefore(element, ref);
      },

      /**
       * Parses function arguments following passed specs.
       *
       * @param args
       * @param {Array} specs
       *   Each item is an array with the following items:
       *   - 0: name of the argument in results object
       *   - 1: function name of the Contextly.Utils object
       *   - 2: boolean flag saying whether the argument is required
       *   - 3: default value in case non-required argument is not found.
       */
      parseArgs: function(args, specs) {
        var params = {};
        var index = 0;
        for (var i = 0; i < specs.length; i++) {
          var spec = specs[i];
          var name = spec[0];
          var candidate = args[index];
          if (candidate == null || !Contextly.Utils[spec[1]](candidate)) { // 1 - check name
            if (spec[2]) { // 2 - is required flag
              Contextly.Utils.error('Required parameter ' + name + ' not found.');
            }

            var fallback = spec[3];
            if (fallback != null) {
              params[name] = fallback;
            }
          }
          else {
            params[name] = candidate;
            index++;
          }
        }
        return params;
      },

      resolveUrlProtocol: function(url, isHttps) {
        if (isHttps == null) {
          isHttps = Contextly.Settings.isHttps();
        }

        if (url.substr(0, 2) === '//') {
          url = 'http' + (isHttps ? 's' : '') + ':' + url;
        }

        return url;
      },

      trim: function(text) {
        // Different versions of jQuery may give inconsistent results on IE8, so
        // we use some well working version to get consistent results.
        return (text || "")
          .replace(/^(\s|\u00A0)+|(\s|\u00A0)+$/g, "");
      }

    }

  });

})();