Contextly.loader = Contextly.loader || {};

Contextly.loader.Controller = Contextly.createClass({

  statics: {

    /**
     * Executes command as soon as the loader is ready.
     *
     * @param {String|Function} command
     *   Provide function to execute when the loader is ready. In case a string
     *   is provided, it may be one of the following function names:
     *   - widgets
     *   - load
     *   - onload
     *   - assets
     *   - libraries
     *   - expose
     *   For the list of arguments see corresponding functions.
     * @param {...*} [args]
     *   The rest arguments are passed to the operation.
     */
    ready: function(command) {
      if (!Contextly.Utils.isFunction(command)) {
        var allowed = {
          widgets: true,
          load: true,
          onload: true,
          assets: true,
          libraries: true,
          expose: true
        };
        if (!(command in allowed)) {
          Contextly.Utils.error('Unknown loader command ' + command);
        }

        command = this[command];
      }

      var args = Array.prototype.slice.call(arguments, 1);
      command.apply(this, args);
    },

    /**
     * Load package and execute function on loading complete.
     *
     * @param {String} [name]
     *   Package name, defaults to "widgets".
     * @param {Function} [callback]
     *   Callback to execute.
     */
    load: function() {
      var args = Contextly.Utils.parseArgs(arguments, [
        ['name', 'isString', false, 'widgets'],
        ['callback', 'isFunction', false]
      ]);

      var name = args.name;
      var gate = new Contextly.CallbacksGate();
      if (args.callback) {
        gate.appendResult(args.callback);
      }

      Contextly.loader.PackageManager.load(name, gate);
      gate.unseal();
    },

    /**
     * Adds function to execute when package is loaded.
     *
     * Callbacks will only be executed in case they were added before the first
     * attempt to load the package.
     *
     * @param {String} [name]
     *   Name of the package to wait for. Defaults to "widgets".
     * @param {Function} callback
     *   Function to execute.
     */
    onload: function() {
      var args = Contextly.Utils.parseArgs(arguments, [
        ['name', 'isString', false, 'widgets'],
        ['callback', 'isFunction', true]
      ]);
      Contextly.loader.PackageManager.addCallback(args.name, args.callback);
    },

    /**
     * Exclude or re-enable assets.
     *
     * @param {Object} options
     *   First level keys are either "css" or "js". Second level keys are
     *   CSS/JS path, values are boolean flags whether to load it or not.
     *   Second level key "*" means all assets of that type.
     */
    assets: function() {
      var args = Contextly.Utils.parseArgs(arguments, [
        ['options', 'isObject', true]
      ]);
      Contextly.loader.PackageManager.setOptions(args.options);
    },

    /**
     * Exclude or re-enable packages.
     *
     * @param {Object} libs
     *   Keys are package names, values may be:
     *   - boolean to specify whether to load it or not
     *   - string "auto" to auto-detect.
     *   Key "*" means all of the packages.
     */
    libraries: function() {
      var args = Contextly.Utils.parseArgs(arguments, [
        ['libs', 'isObject', true]
      ]);
      Contextly.loader.LibrariesManager.setOptions(args.libs);
    },

    /**
     * Adds packages to the loader.
     *
     * @param {Object} packages
     *   Keys are package names, values are package configs.
     */
    expose: function() {
      var args = Contextly.Utils.parseArgs(arguments, [
        ['packages', 'isObject', true]
      ]);
      Contextly.loader.PackageManager.exposePackages(args.packages);
    },

    /**
     * Display or remove widgets.
     *
     * Should be manually called when new posts appear on the page or HTML5
     * History API is used to handle navigation. All arguments are optional and
     * by default it loads "widgets" package and adds them to the whole document.
     *
     * @param {String} [command]
     *   Command to pass to the Contextly.widget.Controller.command() when the
     *   package is loaded.
     * @param {String} [widgets]
     *   Package name to load before calling them. May be used to wait also for
     *   custom foreign packages to load before calling anything.
     * @param {Object|Array} [options]
     *   Array of options passed to Contextly.widget.Controller.command().
     */
    widgets: function() {
      var args = Contextly.Utils.parseArgs(arguments, [
        ['command', 'isString', false, 'add'],
        ['package', 'isString', false, 'widgets'],

        // Since array is also an object, we support both here.
        ['options', 'isObject', false, {}]
      ]);

      this.load(args.package, function() {
        Contextly.widget.Controller.command(args.command, args.options);
      });
    },

    init: function(name) {
      // Simulate package loading complete.
      name = name || 'loader';
      Contextly.assets[name] = Contextly.assets[name] || {};
      Contextly.loader.PackageManager.onPackageLoaded(name);

      // Replace placeholder function with the real one after the package
      // loading simulated, so that handlers couldn't execute anything before
      // pending items processed.
      Contextly.ready = function() {
        var c = Contextly.loader.Controller;
        c.ready.apply(c, arguments);
      };

      // Process pending items in queue. Behavior here is the following:
      // - no queue => nothing is loaded by default;
      // - empty queue => pre-load widgets package, but don't run anything;
      // - any other content => call Contextly.loader.Controller.push() with these
      //   arguments.
      if (Contextly.q && Contextly.Utils.isArray(Contextly.q)) {
        if (!Contextly.q.length) {
          Contextly.q.push(['load', 'widgets', function() {}]);
        }

        while (Contextly.q.length) {
          Contextly.ready.apply(Contextly, Contextly.q.shift());
        }
      }
    }

  }

});
