(function($) {

  Contextly.widget.Post = Contextly.createClass({

    extend: [Contextly.ProxyEach, Contextly.Transmitter],

    construct: function(options) {
      this.options = Contextly.Utils.merge({
        context: document,
        metadata: null,
        data: null,
        update: null,
        guid: null
      }, options);

      this.queue = [
        this.init,
        this.loadData,
        this.convertData,
        this.setApiSessionId,
        this.buildContainers,
        [this.loadPackages, this.loadFonts],
        this.buildWidgets,
        this.displayWidgets,
        this.scheduleUpdate
      ];
      this.load();
    },

    load: function() {
      if (!this.queue.length || this.gate) {
        return;
      }

      var actions = this.queue.shift();
      if (!Contextly.Utils.isArray(actions)) {
        actions = [actions];
      }

      this.gate = new Contextly.CallbacksGate();
      this.each(actions, function(action) {
        action.call(this);
      });
      if (!this.isDestroyed()) {
        this.gate.appendResult(this.proxy(this.onStageComplete));
        this.gate.unseal();
      }
    },

    onStageComplete: function() {
      delete this.gate;
      this.load();
    },

    init: function() {
      this.$context = $(this.options.context);
      this.factory = new Contextly.widget.Factory(this.$context);

      if (this.options.metadata != null) {
        this.metadataProvider = new Contextly.metadataProvider.Custom(this.options.metadata);
      }
      else {
        this.metadataProvider = new Contextly.metadataProvider.Parsed();
      }
      if (!this.metadataProvider.hasRequired()) {
        Contextly.Utils.logError('Required metadata properties not found in context:', this.$context);
        this.destroy();
        return;
      }

      this.apiSession = new Contextly.client.Session();
      this.restClient = new Contextly.client.Rest(this.apiSession, this.metadataProvider);
      this.experienceClient = new Contextly.client.Experience(this.apiSession, this.metadataProvider);
      this.mainServerClient = new Contextly.client.MainServer();
      this.eventsLogger = new Contextly.EventsLogger(this.experienceClient);

      // Allow caller to override data we retrieve from external sources.
      var keys = {
        data: true,
        update: true,
        guid: true
      };
      for (var key in keys) {
        if (this.options[key] != null) {
          this[key] = this.options[key];
        }
      }
    },

    getDestroyableProperties: function() {
      return [
        'gate',
        'eventsLogger',
        'restClient',
        'experienceClient',
        'mainServerClient',
        'metadataProvider',
        'factory'
      ];
    },

    destroyReferences: function() {
      this.queue = [];

      if (this.widgets) {
        this.each(this.widgets, function(widget) {
          widget.removeCssCode();
          widget.destroy();
        });
        delete this.widgets;
      }

      Contextly.ProxyEach.fn.destroyReferences.apply(this, arguments);

      this.broadcast(Contextly.widget.Post.broadcastTypes.DESTROYED, this.$context);
      delete this.$context;
    },

    getWidgetLoadingFlags: function() {
      var result = {};

      var types = Contextly.widget.types;
      var map = {
        load_snippet: types.SNIPPET,
        load_side_rail: types.SIDERAIL,
        load_sidebar: types.SIDEBAR,
        load_auto_sidebar: types.AUTO_SIDEBAR,
        load_storyline_subscribe: types.STORYLINE_SUBSCRIBE,
        load_personalization_subscribe: types.PERSONALIZATION_SUBSCRIBE,
        load_channel_subscribe: types.CHANNEL_SUBSCRIBE,
        load_social: types.SOCIAL
      };
      var fallbacks = {
        load_auto_sidebar: 'auto',
        load_channel_subscribe: 'auto'
      };

      for (var flag in map) {
        var type = map[flag];
        var value = !!this.factory.getContainer(type)
          .getElements()
          .length;
        if (!value && fallbacks[flag] != null) {
          value = fallbacks[flag];
        }
        result[flag] = value;
      }

      return result;
    },

    loadData: function() {
      if (this.data != null) {
        return;
      }

      this.broadcast(Contextly.widget.Post.broadcastTypes.LOADING, this.$context);

      var reason = this.gate.addReason();
      var params = this.getWidgetLoadingFlags();
      this.restClient.call('pagewidgets', 'get', params, this.proxy(function(response) {
        this.onDataLoadingComplete(response);
        reason.call(window);
      }, false, true));
    },

    onDataLoadingComplete: function(response) {
      if (response && response.success && response.entry) {
        this.onDataLoadingSuccess(response);
      }
      else {
        this.onDataLoadingError(response);
      }
    },

    onDataLoadingSuccess: function(response) {
      this.broadcast(Contextly.widget.Post.broadcastTypes.LOADED, this.$context, response);

      Contextly.Visitor.initIds(response);

      this.data = response.entry;
      if (this.update == null && response.entry.update != null) {
        this.update = response.entry.update;
      }
      if (this.guid == null && response.guid != null) {
        this.guid = response.guid;
      }
    },

    onDataLoadingError: function(response) {
      this.broadcast(Contextly.widget.Post.broadcastTypes.FAILED, this.$context, response);
      this.destroy();
    },

    convertData: function() {
      var result = [];
      this.each(this.data, function(collection) {
        if (!Contextly.Utils.isArray(collection) || !collection.length) {
          return;
        }

        this.each(collection, function(data) {
          if (!data || !data.type) {
            return;
          }

          result.push(data);
        });
      });
      this.data = result;
    },

    setApiSessionId: function() {
      if (this.apiSession.hasId()) {
        return;
      }

      this.apiSession.setId(this.guid);
    },

    buildContainers: function() {
      this.containers = [];
      this.each(this.data, function(data) {
        var container = this.factory.getContainer(data.type, data);
        if (container) {
          this.containers.push(container);
        }
      });
    },

    loadPackages: function() {
      this.each(this.containers, function(container) {
        var widgetPackages = {
          stylist: container.getStylistPackageName(),
          theme: container.getThemePackageName(),
          widget: container.getWidgetPackageName()
        };
        for (var key in widgetPackages) {
          var name = widgetPackages[key];
          if (name != null) {
            Contextly.loader.PackageManager.load(name, this.gate);
          }
        }
      });
    },

    loadFonts: function() {
      this.each(this.containers, function(container) {
        var data = container.getData();
        var fonts = data.settings.external_fonts;
        if (fonts) {
          Contextly.FontManager.load({
            fonts: fonts,
            callback: this.gate.addReason()
          });
        }
      });
    },

    buildWidgets: function() {
      this.widgets = [];
      var commonParams = {
        eventsLogger: this.eventsLogger,
        experienceClient: this.experienceClient,
        mainServerClient: this.mainServerClient
      };
      this.each(this.containers, function(container) {
        var params = Contextly.Utils.merge({
          data: container.getData(),
          container: container
        }, commonParams);
        this.widgets.push(this.factory.getWidget(params));
      });
    },

    displayWidgets: function() {
      this.each(this.widgets, function(widget) {
        widget.loadCssCode();
        widget.display();
      });

      this.broadcast(Contextly.widget.Post.broadcastTypes.DISPLAYED, this.$context, this.widgets);
    },

    scheduleUpdate: function() {
      if (!this.update) {
        return;
      }

      // Allow listeners to postpone the update using the gate.
      var gate = new Contextly.CallbacksGate();
      gate.appendResult(this.proxy(function() {
        this.restClient.call('postsimport', 'put');
      }));
      var broadcastType = Contextly.widget.Post.broadcastTypes.POST_UPDATE;
      this.broadcast(broadcastType, this.$context, this.metadataProvider, gate);
      gate.unseal();
    }

  });

  // Broadcast event types.
  Contextly.widget.Post.broadcastTypes = {
    LOADING: 'contextlyWidgetsLoading',
    LOADED: 'contextlyWidgetsLoaded',
    FAILED: 'contextlyWidgetsFailed',
    DISPLAYED: 'contextlyWidgetsDisplayed',
    DESTROYED: 'contextlyWidgetsDestroyed',
    POST_UPDATE: 'contextlyPostUpdate'
  };

})(jQuery);