(function($) {

  /**
   * @class
   */
  Contextly.widget.TweetsManager = Contextly.createClass({

    extend: Contextly.Proxy.prototype,

    statics: {

      construct: function() {
        this.handlers = {};
        this.binded = {};
      },

      /**
       * Binds twitter events.
       *
       * Should only be called AFTER twitter library reported it's ready.
       *
       * @param {Contextly.widget.Base} widget
       * @param {object} events
       *   Keys are event types, values are either single callback or array of
       *   callbacks.
       */
      bind: function(widget, events) {
        var index = widget.getWidgetIndex();
        for (var type in events) {
          this.handlers[type] = this.handlers[type] || {};
          this.handlers[type][index] = this.handlers[type][index] || [];

          var handlers = events[type];
          if (!Contextly.Utils.isArray(handlers)) {
            handlers = [handlers];
          }
          Array.prototype.push.apply(this.handlers[type][index], handlers);

          if (!this.binded[type]) {
            twttr.events.bind(type, this.proxy(this.onEvent, false, true));
            this.binded[type] = true;
          }
        }
      },

      onEvent: function(e) {
        var $container = this.getTweetContainer(e.target);
        if (!$container.length) {
          return;
        }

        var index = $container.data('ctxWidgetIndex');
        if (index == null) {
          return;
        }

        if (!this.handlers[e.type] || !this.handlers[e.type][index]) {
          return;
        }

        var callbacks = this.handlers[e.type][index];
        for (var i = 0; i < callbacks.length; i++) {
          callbacks[i].call(this, e, $container);
        }
      },

      /**
       * Unbind tweet events for the widget.
       *
       * @param widget
       * @param [types]
       *   Keys are event types, values are ignored. All types will be detached
       *   in case types list is omitted.
       */
      unbind: function(widget, types) {
        types = types || this.handlers;
        var index = widget.getWidgetIndex();

        for (var type in types) {
          if (!this.handlers[type] || !this.handlers[type][index]) {
            continue;
          }

          delete this.handlers[type][index];
        }
      },

      getTweetContainer: function(element) {
        return $(element)
          .parents('.ctx-tweet:first');
      }

    }

  });

  /**
   * @mixin
   */
  Contextly.widget.TweetsRenderer = Contextly.createClass({

    queueTweetsRendering: function() {
      twttr.ready(this.proxy(this.onTwitterReady));
    },

    /**
     * @function
     * @param widgetElements
     */
    getTweetContainers: Contextly.abstractMethod(),

    onTwitterReady: function() {
      this.bindTwitterEvents();
      this.renderTweets(this.getWidgetElements());
    },

    renderTweets: function(widgetElements) {
      var tweetContainers = this.getTweetContainers(widgetElements);
      if (!tweetContainers.length) {
        return;
      }

      // Twitter event handlers are global and we should check whether the
      // event was triggered on tweet inside current widget, so we mark all
      // the containers with the widget index.
      this.markWithWidgetIndex(tweetContainers);

      this.eachElement(tweetContainers, this.renderTweet);
    },

    renderTweet: function(element) {
      var id = element.attr('data-tweet-id');
      twttr.widgets.createTweet(id, element[0], {
        cards: 'hidden',
        conversation: 'none'
      });
    },

    bindTwitterEvents: function() {
      // Bind the events provided by Twitter.
      var handlers = {
        rendered: this.proxy(this.onTweetRendered, false, true)
      };

      // We don't bind "like" type yet, because Twitter triggers both like and
      // favourite events in this case.
      var types = ['click', 'tweet', 'follow', 'retweet', 'favorite'];
      var handler = this.proxy(this.onTweetInteraction, false, true);
      this.each(types, function(eventName) {
        handlers[eventName] = handler;
      });

      Contextly.widget.TweetsManager.bind(this, handlers);
    },

    /**
     * Binds an event to the DOM element inside the Tweet.
     *
     * We have to re-invent the wheel because some jQuery versions may fail with
     * binding events to a shadow DOM.
     *
     * @param $container
     * @param element
     * @param eventName
     * @param callback
     *   Automatically proxied with both context and arguments passed.
     */
    bindTweetElementEvent: function($container, element, eventName, callback) {
      var proxiedCallback = this.proxy(function(e) {
        return callback.call(this, e, $container, element);
      }, false, true);

      if (element.addEventListener) {
        element.addEventListener(eventName, proxiedCallback);
      }
      else {
        Contextly.Utils.error("Browser doesn't support basic event binding.");
      }
    },

    /**
     * @param $container
     *   Tweet container element.
     * @param tweetDoc
     *   Either frame document or shadow root of the tweet.
     */
    applyTweetMods: function($container, tweetDoc) {
      if (!tweetDoc) {
        return;
      }

      try {
        // Bind tweet click events as soon as the tweet rendering is
        // complete, because twitter doesn't provide events here.
        var blockquote = tweetDoc.querySelector('blockquote');
        if (blockquote) {
          this.bindTweetElementEvent($container, blockquote, 'click', this.onTweetClick);
        }

        // Cancel clicks on tweet & change the cursor. Since Twitter library
        // binds event handlers to both document and the embedded tweet div
        // we have to use event capturing here and catch the click event on
        // the iframe document to cancel it.
        var embeddedTweet = tweetDoc.querySelector('div.EmbeddedTweet');
        if (embeddedTweet && embeddedTweet.style) {
          embeddedTweet.style.cursor = 'default';
        }
        if (tweetDoc.addEventListener) {
          tweetDoc.addEventListener('click', this.proxy(function(e) {
            var targetA = this.findClosestTag(e.target, 'a');
            if (targetA == null) {
              e.stopPropagation();
            }
          }, false, true), true);
        }
      }
      catch (err) {
        Contextly.Utils.logError('Unable to apply tweet mods.', err);
      }
    },

    onTweetRendered: function(e, $link) {
      var $target = $(e.target);

      if ($target.is('iframe')) {
        this.applyTweetMods($link, $target.contents()[0]);
      }
      else if ($target.is('twitterwidget') && e.target.shadowRoot) {
        this.applyTweetMods($link, e.target.shadowRoot);
      }
      else {
        Contextly.Utils.logError('Unable to detect the type of tweet.');
      }

      // Trigger custom event on the tweet container.
      $link.triggerHandler('contextlyTweetRendered');
    },

    /**
     * Finds closest tag with passed tag name among element itself or its parents.
     *
     * @param element
     * @param tagName
     *   Tag name in lower case.
     */
    findClosestTag: function(element, tagName) {
      while (element.parentNode != null && element.tagName != null) {
        if (element.tagName.toLowerCase() === tagName) {
          return element;
        }
        else {
          element = element.parentNode;
        }
      }

      return null;
    },

    onTweetClick: function(e, $container, blockquote) {
      // Find target A tag.
      var targetA = this.findClosestTag(e.target, 'a');
      if (!targetA) {
        return;
      }

      // Check and extract A scribe type.
      var scribeType = targetA.getAttribute('data-scribe');
      if (scribeType == null || !scribeType.match(/^element\:/)) {
        return;
      }

      scribeType = scribeType.replace(/^element\:/, '');
      if ($.inArray(scribeType, ['reply', 'retweet', 'favorite', 'heart']) !== -1) {
        // These types are handled through native Twitter events.
        return;
      }

      var tweetId = blockquote.getAttribute('data-tweet-id');
      if (!tweetId) {
        return;
      }

      var href = targetA.getAttribute('href');
      if (scribeType === 'url') {
        var expandedUrl = targetA.getAttribute('data-expanded-url');

        if (expandedUrl) {
          // Use expanded rather than shortened URL on the events log.
          href = expandedUrl;

          // On the target URL we should send a request to the main server and
          // exit.
          var nativeUrl = $container.attr('data-native-url');
          if (expandedUrl.toLowerCase() === nativeUrl.toLowerCase()) {
            if (!this.onTweetTargetUrlClick(e, $container)) {
              return;
            }
          }
        }
      }

      this.logTweetEvent('link', this.alterLoggedEventParams({
        event_element: scribeType,
        event_tweet_id: tweetId,
        event_url: href
      }));
    },

    onTweetTargetUrlClick: function(e, $tweetContainer) {
      // Nothing by default.
      return true;
    },

    onTweetInteraction: function(e, $link) {
      var tweetId = $link.attr('data-tweet-id');
      if (!tweetId) {
        return;
      }

      this.logTweetEvent(e.type, this.alterLoggedEventParams({
        event_element: e.region,
        event_tweet_id: tweetId
      }));
    },

    logTweetEvent: function(name, params) {
      params = this.alterLoggedEventParams(params);

      this.eventsLogger.tweetEvent(name, params);
    },

    destroyReferences: function() {
      Contextly.widget.TweetsManager.unbind(this);
    }

  });

})(jQuery);