(function($) {

  /**
   * Base abstract class for all widgets.
   *
   * @class
   * @extends Contextly.ProxyEach
   */
  Contextly.widget.Base = Contextly.createClass( /** @lends Contextly.widget.Base.prototype */ {

    extend: [Contextly.ProxyEach, Contextly.Transmitter],

    construct: function(params) {
      this.widget = params.data;
      this.containers = params.container;
      this.eventsLogger = params.eventsLogger;
      this.experienceClient = params.experienceClient;
      this.mainServerClient = params.mainServerClient;

      this.widget_elements = null;
      this.widget_index = ++Contextly.widget.index;
    },

    getDestroyableProperties: function() {
      return [
        "mainServerClient",
        "experienceClient",
        "eventsLogger",
        "containers",
        "widget"
      ];
    },

    destroyReferences: function() {
      this.remove();

      Contextly.ProxyEach.fn.destroyReferences.apply(this, arguments);
    },

    getWidget: function() {
      return this.widget;
    },

    getWidgetType: function() {
      return this.widget_type;
    },

    getSettings: function() {
      return this.getWidget().settings;
    },

    getWidgetIndex: function() {
      return this.widget_index;
    },

    markWithWidgetIndex: function(elements) {
      $(elements)
        .data('ctxWidgetIndex', this.getWidgetIndex());
    },

    getWidgetStyleClass: function() {
      return 'default-widget';
    },

    getWidgetElements: function() {
      if (this.widget_elements === null) {
        this.widget_elements = this.getWidgetContainers()
          .find('.' + this.getWidgetStyleClass());
      }

      return this.widget_elements;
    },

    getWidgetContainers: function() {
      return this.containers.getElements();
    },

    getEventsNamespace: function() {
      return 'contextlyWidget' + this.getWidgetIndex();
    },

    /**
     * Adds namespace to the passed jQuery event type.
     */
    nsEvent: function(type) {
      var ns = '.' + this.getEventsNamespace();

      if (type) {
        type = type.split(/\s+/)
          .join(ns + ' ');
      }
      else {
        type = '';
      }
      type += ns;

      return type;
    },

    /**
     * @function
     */
    getWidgetHTML: Contextly.abstractMethod(),

    // TODO Replace use cases with template and drop.
    displayHTML: function (html) {
      this.getWidgetContainers().html(html);
    },

    // TODO Replace use cases with template and drop.
    appendHTML: function(html) {
      this.getWidgetContainers().append(html);
    },

    display: function() {
      var widget_html = this.getWidgetHTML();
      this.displayHTML(widget_html);
      this.attachHandlers();
      this.broadcastWidgetDisplayed();
    },

    remove: function() {
      this.getWidgetContainers()
        .empty();

      this.broadcast(Contextly.widget.broadcastTypes.REMOVED);
    },

    attachHandlers: function() {
      var handlers = this.getHandlers.apply(this, arguments);
      for (var name in handlers) {
        this[name].call(this);
      }
    },

    getHandlers: function() {
      return {};
    },

    /**
     * @param type
     * @param {...*}
     *   The rest arguments are passed to the handler after widget type and
     *   current instance of {Contextly.widget.Base}.
     */
    broadcast: function(type) {
      var args = [type, this.containers.getContext(), this.widget_type];

      // Pass arguments after type.
      if (arguments.length > 1) {
        Array.prototype.push.apply(args, Array.prototype.slice.call(arguments, 1));
      }

      Contextly.Transmitter.fn.broadcast.apply(this, args);
    },

    broadcastWidgetDisplayed: function() {
      this.broadcast(Contextly.widget.broadcastTypes.DISPLAYED);
    },

    getCssCodeKey: function() {
      return 'widgets/settings:' + this.getWidgetType();
    },

    loadCssCode: function() {
      var key = this.getCssCodeKey();
      var generator = this.proxy(this.getCustomCssCode);
      Contextly.loader.CssManager.loadCode(key, generator);
    },

    removeCssCode: function() {
      Contextly.loader.CssManager.removeCode(this.getCssCodeKey());
    },

    getCustomCssCode: function() {
      // Do nothing by default.
    },

    // TODO Check if any use cases left after moving to templates and drop.
    escape: function(text) {
      return Contextly.Utils.escape(text);
    },

    /**
     * Renders attributes with escaped values surrounded by double quotes.
     *
     * Attribute names are *not escaped*!
     */
    renderAttributes: function(attributes) {
      var res = '';
      for (var key in attributes) {
        res += ' ' + key + '="' + Contextly.Utils.escape(attributes[key]) + '"';
      }
      return res;
    },

    logEvent: function(name, params) {
      params = this.alterLoggedEventParams(params);

      this.eventsLogger.logEvent(name, params);
    },

    alterLoggedEventParams: function(params) {
      params.event_widget_type = this.getWidgetType();
      return params;
    }

  });

})(jQuery);
