(function($) {

  /**
   * @class
   * @extends Contextly.widget.Base
   * @extends Contextly.widget.TweetsRenderer
   * @extends Contextly.widget.ViewHandler
   * @extends Contextly.widget.ResponsiveLayout
   */
  Contextly.widget.Social = Contextly.createClass( /** @lends Contextly.widget.Social.prototype */ {

    extend: [
      Contextly.widget.Base,
      Contextly.widget.TweetsRenderer,
      Contextly.widget.ViewHandler,
      Contextly.widget.ResponsiveLayout
    ],

    construct: function(params) {
      Contextly.widget.Base.apply(this, arguments);

      this.slicks = [];
      this.widget_type = Contextly.widget.types.SOCIAL;
      this.widget_displayed = false;
      this.first_tweet_rendered = false;
    },

    destroyReferences: function() {
      if (this.slicks.length) {
        this.getWidgetElements()
          .find('.ctx-tweets')
          .unbind('refresh')
          .unbind('beforeRefresh')
          .unbind('beforeChange');

        this.each(this.slicks, function(slick) {
          slick.destroy();
        });
      }
      this.slicks = [];

      Contextly.widget.ResponsiveLayout.fn.destroyReferences.apply(this, arguments);
      Contextly.widget.TweetsRenderer.fn.destroyReferences.apply(this, arguments);
      Contextly.widget.ViewHandler.fn.destroyReferences.apply(this, arguments);
      Contextly.widget.Base.fn.destroyReferences.apply(this, arguments);
    },

    hasWidgetData: function() {
      var links = this.getWidget().links;
      return links && links.length;
    },

    display: function() {
      if (this.hasWidgetData()) {
        // Rendering is done in three steps:
        // 1. Wait for the Twitter library to load.
        // 2. Render HTML, init Slick, render tweets, wait for all tweets to
        //    load for the first time.
        // 3. On all tweets displayed for the first time broadcast the widget
        //    displayed event.
        this.queueTweetsRendering();
      }
      else {
        this.getWidgetContainers().empty();
        this.broadcastWidgetDisplayed();
      }
    },

    onTwitterReady: function() {
      // Rendering phase 2. Display outer HTML, bind Twitter events, set up
      // Slick,
      var widget_html = this.getWidgetHTML();
      this.displayHTML(widget_html);
      this.setUpResponsiveLayout();
      this.bindTwitterEvents();
      this.setUpSlick();

      this.tweets_rendering_gate = new Contextly.CallbacksGate();
      this.tweets_rendering_gate.appendResult(this.proxy(this.onAllTweetsRendered));
      this.renderTweets(this.getWidgetElements());
      this.tweets_rendering_gate.unseal();
    },

    /**
     * Attaches event handlers on the first tweet rendering.
     *
     * There is no sense in handling this event per widget instance, as we just
     * wait for the data to start arriving.
     */
    onFirstTweetRendered: function() {
      if (this.first_tweet_rendered) {
        return;
      }
      this.first_tweet_rendered = true;

      this.attachWidgetViewHandler();
      this.attachNavigationEvents();
      this.handleHashNavigation();
    },

    onAllTweetsRendered: function() {
      if (!this.widget_displayed) {
        this.widget_displayed = true;
        this.broadcastWidgetDisplayed();
      }
    },

    handleHashNavigation: function() {
      // We parse the hash for existence of our anchor, as we may add it to an
      // existing URL hash.
      var hash = window.location.hash;
      if (hash.match(/(?:^|#)ctx\-onr(?:$|#)/i)) {
        this.scrollToFirstWidget();
      }
    },

    scrollToFirstWidget: function() {
      var firstElement = this.getWidgetElements().eq(0);
      if (!firstElement.length) {
        return;
      }

      var elementTop = Math.max(0, firstElement.offset().top - 30);

      // Weird selector here is because we can't animate it on window and the
      // actual element is browser-specific.
      $('html, body').animate({
        scrollTop: elementTop
      });
    },

    getWidgetStyleClass: function() {
      return 'ctx-social-' + this.getSettings().theme;
    },

    getWidgetHTML: function() {
      var html = '';
      var s = this.getSettings();

      if (s.title) {
        html += "<div class='ctx-title'><p>" + this.escape(s.title) + "</p></div>";
      }

      html += "<div class='ctx-content ctx-clearfix'>"
        + "<div class='ctx-tweets'>"
        + this.getLinksHTML()
        + "</div>"
        + "</div>";

      var classes = [
        'ctx-social-widget',
        this.getWidgetStyleClass()
      ];
      return "<div class='" + this.escape(classes.join(' ')) + "'>" + html + "</div>";
    },

    getLinkHTML: function(link) {
      var attributes = this.renderAttributes({
        "class": "ctx-tweet ctx-icon ctx-icon-hourglass",
        "data-tweet-id": link.tweet,
        "data-native-url": link.native_url,
        "data-contextly-url": link.url
      });

      return '<div' + attributes + '></div>';
    },

    getLinksHTML: function() {
      var html = '';

      if (this.hasWidgetData()) {
        var links = this.getWidget().links;
        for (var i = 0; i < links.length; i++) {
          html += this.getLinkHTML(links[i]);
        }
      }

      return html;
    },

    getLayoutModes: function() {
      return {
        "mobile": [0, 350],
        "1-tweet": [350, 1040],
        "2-tweets": [1040, 1510],
        "3-tweets": [1510, 1980],
        "wide": [1980]
      };
    },

    setLayoutMode: function() {
      var res = Contextly.widget.ResponsiveLayout.fn.setLayoutMode.apply(this, arguments);

      // Because we also postpone mode update and Slick doesn't have a chance
      // to react, we must trigger its resize manually.
      this.each(this.slicks, function(slick) {
        slick.checkResponsive();
      });

      return res;
    },

    setUpSlick: function() {
      this.eachElement(this.getWidgetElements(), function(widgetElement) {
        var slickElement = widgetElement.find('.ctx-tweets');
        var data = {
          widgetElement: widgetElement
        };

        // Since on Slick breakpoint switch all the tweets loose their content
        // because of DOM manipulations made by the library, we have to handle
        // it in the following way:
        // - Before the refresh we drop frames of the tweets to make the switch
        //   much more lightweight.
        // - After the refresh we just re-load all the tweets.
        callback = this.proxy(this.onSlickBeforeRefresh, false, true);
        slickElement.bind(this.nsEvent('beforeRefresh'), data, callback);
        var callback = this.proxy(this.onSlickAfterRefresh, false, true);
        slickElement.bind(this.nsEvent('refresh'), data, callback);

        this.slicks.push(new Contextly.Slick(slickElement, {
          infinite: true,
          slidesToShow: 4,
          slidesToScroll: 1,
          touchMove: false,
          draggable: false,
          centerMode: true,
          centerPadding: '50px',
          zIndex: 5,
          respondTo: 'slider',
          responsive: [
            {
              breakpoint: 1511,
              settings: {
                slidesToShow: 3
              }
            },
            {
              breakpoint: 1041,
              settings: {
                slidesToShow: 2
              }
            },
            {
              breakpoint: 571,
              settings: {
                slidesToShow: 1
              }
            },
            {
              breakpoint: 351,
              settings: {
                slidesToShow: 1,
                centerPadding: '8px'
              }
            }
          ]
        }));
      });
    },

    onSlickBeforeRefresh: function(e) {
      if (!e.data || !e.data.widgetElement) {
        return;
      }

      this.getTweetContainers(e.data.widgetElement)
        .empty()
        .addClass('ctx-icon ctx-icon-hourglass');
    },

    onSlickAfterRefresh: function(e) {
      if (!e.data || !e.data.widgetElement) {
        return;
      }

      this.renderTweets(e.data.widgetElement);
    },

    buildLayoutClass: function(mode) {
      return 'ctx-social-' + mode;
    },

    getCustomCssCode: function() {
      return Contextly.widget.SocialCssCustomBuilder
        .buildCSS('.' + this.getWidgetStyleClass(), this.getSettings());
    },

    getTweetContainers: function(widgetElements) {
      return widgetElements.find('.ctx-tweet[data-tweet-id]');
    },

    onTweetTargetUrlClick: function(e, $tweetContainer) {
      var contextlyUrl = $tweetContainer.attr('data-contextly-url');
      this.mainServerClient.call(contextlyUrl);

      // Prevent logging it as an ordinary click.
      return false;
    },

    applyTweetMods: function($container, tweetDoc) {
      Contextly.widget.TweetsRenderer.fn.applyTweetMods.apply(this, arguments);

      // Workaround to the Twitter library issue on Firefox & IE9. Tweets
      // appear cropped on these browsers, but as soon as we try to resize
      // them the Twitter library does its job properly and fixes the size.
      var $iframe = $container.find('iframe.twitter-tweet');
      if ($iframe.length) {
        var $html = $(tweetDoc).find('html:first');
        if ($html.length) {
          var height = $html.height();
          if (height > 0) {
            $iframe.height(height);
          }
        }
      }
    },

    renderTweet: function(element) {
      element
        .unbind('contextlyTweetRendered')
        .one('contextlyTweetRendered', this.proxy(function(el) {
          $(el).removeClass('ctx-icon ctx-icon-hourglass');
        }, true));

      if (!this.tweets_rendering_gate.isComplete()) {
        element.one('contextlyTweetRendered', this.tweets_rendering_gate.addReason());
      }

      if (!this.first_tweet_rendered) {
        element.one('contextlyTweetRendered', this.proxy(this.onFirstTweetRendered));
      }

      Contextly.widget.TweetsRenderer.fn.renderTweet.apply(this, arguments);
    },

    findViewWatcherElement: function(widgetElement) {
      return widgetElement.find('.ctx-tweets');
    },

    attachNavigationEvents: function() {
      this.getWidgetElements()
        .find('.ctx-tweets')
        .bind(this.nsEvent('beforeChange'), this.proxy(this.onSlickNavigation, false, true));
    },

    onSlickNavigation: function(e, slick, prev, next) {
      var diff = Math.abs(prev - next);

      if (!diff) {
        // Navigation without diff happens on reaching responsive layout
        // breakpoint, we just ignore it.
        return;
      }

      if (diff === 1) {
        if (next > prev) {
          this.onSlickNext();
        }
        else {
          this.onSlickPrev();
        }
      }
      else if (this.hasWidgetData()) {
        var linksNum = this.getWidget().links.length;
        if (prev === 0 && next === linksNum - 1) {
          this.onSlickPrev();
        }
        else if (next === 0 && prev === linksNum - 1) {
          this.onSlickNext();
        }
      }
    },

    onSlickPrev: function() {
      this.logEvent(Contextly.widget.eventNames.CAROUSEL_NAVIGATION, {
        event_direction: 'previous'
      });
    },

    onSlickNext: function() {
      this.logEvent(Contextly.widget.eventNames.CAROUSEL_NAVIGATION, {
        event_direction: 'next'
      });
    }

  });

})(jQuery);
